#ifndef SPHERE_H
#define SPHERE_H

#include <Vector3D.h>
#include "../Interfaces/PrimitiveInterface.h"
#include "../Interfaces/TextureInterface.h"
#include "../Textures/SimpleColorTexture.h"


/// A sphere, specified by a center and a radius
class Sphere : public PrimitiveInterface{
public :

  /// General constructor for Sphere
  /**
   * @param n
   *        the name of the sphere
   * @param c
   *        the center of the <code>Sphere</code>.
   * @param r 
   *        the radius, which must be positive.
   * @param t
   *        the texture carried by this primitive.
  */
  Sphere (QString n, const Vector3D & c = Vector3D (0.0, 0.0, 0.0) , float r = 0.0, 
	  TextureInterface * t = DEFAULT_SIMPLECOLOR);
  
  Sphere (Sphere &s);
  virtual ~Sphere () {}
  
  inline float getRadius () const { return radius;}
  
  inline Vector3D getCenter () const { return center;}
  
  void setRadius (float newRadius); 
  
  inline void setCenter (Vector3D& newCenter) { center = newCenter;}
  
  inline Sphere& operator=(Sphere& sphere) {
    radius = sphere.radius;
    center = sphere.center;
    return (*this);
  }

  // *******************
  // CoTeXNode Interface
  // *******************
  
  /// F-Rep evaluaton of a sphere
  /**
   * @return double the difference between distance center<->position, 
   *                and the sphere's radius.
   */
  double eval (const Vector3D& position);

  BBoxMinMax getBBoxMinMax ();

  // **********************
  // HyperFunable Interface
  // **********************
  
  void writeHFString (QTextStream & outStream);
  void writeDefArray (QTextStream & outStream);
  void writeDefBody (QTextStream & outStream);
 
  // *****************
  // Savable Interface
  // *****************
  
  void save (QTextStream & stream);
  void load (QTextStream & stream);

private:

  float radius;
  Vector3D center;

};
  

#endif // SPHERE_H
