#ifndef PERLINNOISECOLORTEXTURE_H
#define PERLINNOISECOLORTEXTURE_H

#include "../Interfaces/TextureInterface.h"
#include "../Attributes/ColorAttribute.h"
#include <math.h>
#include <qstring.h>

/// The differents types of noise.
typedef enum NoiseType {
  MARBLENOISE = 0,
  WOODNOISE = 1,
  CLOUDNOISE = 2,
  V_SCRATCHED = 3,
  H_SCRATCHED = 4
} NoiseType;

/// Perlin Noise value in the space.
class PerlinNoiseColorTexture : public TextureInterface {
public :
  /// Constructor for the Perlin Noise Texture
  /**
   * @param n
   *       name of the texture
   * @param c1 
   *      first color of the texture
   * @param c2 
   *      second color of the texture
   */
  PerlinNoiseColorTexture (QString n = QString ("PerlinNoise"), 
			   ColorAttribute c1 = ColorAttribute (), 
			   ColorAttribute c2 = ColorAttribute (255, 255, 255),
			   double pha = 0.0,
			   double freq = 2.0,
			   double amp = 8.0,
			   double b1 = 0.0,
			   double b2 = 2.0,
			   NoiseType nType = MARBLENOISE);
  
  ///Destructor 
  virtual ~PerlinNoiseColorTexture () {}
  
  inline ColorAttribute getColor1 () const { return color1;} 
  inline ColorAttribute getColor2 () const { return color2;} 
  inline void setColor1 (int r, int g, int b) { color1.setRgb (r, g, b);} 
  inline void setColor2 (int r, int g, int b) { color2.setRgb (r, g, b);} 
  
  inline double getPhase () const { return phase;}
  inline double getFrequency () const { return frequency;}
  inline double getAmplitude () const { return amplitude;}
  inline double getBound1 () const { return bound1;}
  inline double getBound2 () const { return bound2;}
  inline NoiseType getNoiseType () const { return noiseType;}
 

  // **************************
  // Interface TextureInterface
  // **************************
  
  ///Returns the new color
  /**
   * @param v
   *        the position ...
   * @return an AttributeInterface 
   *         the result color after the application of "Perlin Noise Texture"
   */
  const AttributeInterface * operator() (const Vector3D& v);
 
  // **********************
  // Interface HyperFunable
  // **********************
 
  void writeHFString (QTextStream & outStream);
  void writeDefArray (QTextStream & outStream);
  void writeDefBody (QTextStream & outStream);

  // *****************
  // Interface Savable
  // *****************
  
  void save (QTextStream & stream);
  void load (QTextStream & stream);

private :
  ColorAttribute color1;
  ColorAttribute color2;
  ColorAttribute returnedColor;
 
  float phase, frequency, amplitude;
  float bound1, bound2;
  NoiseType noiseType;
  bool randomColor1, randomColor2;
  double turbulence (float x, float y, float z);
  
  /// Mixes the two colors 
  /**
   * @param c1
   * @param c2
   * @param val
   * @return an ColorAttribute
   */
  void mix (double val);
};

#endif // PerlinNoiseColorTexture_H
