#include "DoubleSpinBox.h"

#include <math.h>

DoubleSpinBox::DoubleSpinBox (double minValue, double maxValue, int decimals, 
			      double step, double defaultValue, QWidget* parent, const char* name)
  : QSpinBox ((int)(minValue * pow (10, decimals)), (int)(maxValue * pow (10, decimals)), 
	      (int)(step * pow (10, decimals)), parent, name), dec (decimals){

  pow10dec = pow (10, dec);
  dVal = new QDoubleValidator (minValue, maxValue, dec, this);
  setValidator (dVal);	
  setValue (defaultValue);
}

QString DoubleSpinBox::mapValueToText (int value) {
  QString s;
  s.setNum ((double)(value / pow10dec ), 'f', dec );
  return s;
}

int DoubleSpinBox::mapTextToValue (bool * ok) {
  return (int)(cleanText ().toDouble (ok) * pow10dec);
}


double DoubleSpinBox::value() const
{
  return (double)(QRangeControl::value() / pow10dec);
}

void DoubleSpinBox::setValue (double value) {
  QSpinBox::setValue (value * pow10dec);
}
